
#pragma once

#include <windows.h>

class CThread 
{
public:
	HANDLE m_hThread;
	DWORD m_dwId;
	bool m_AutoStop;
	bool m_WaitForEnd;
public:
	// Default parameter are set for as Anti-Reversing Module
	CThread(bool autoStart = false, bool autoStop = true, bool waitForEnd = true)
		: m_hThread( NULL )
		, m_dwId()
		, m_AutoStop( autoStop )
		, m_WaitForEnd( waitForEnd )
	{
		if ( autoStart ) {
			Create();
		}
	}

	virtual ~CThread()
	{
		if ( m_AutoStop ) {
			if ( m_WaitForEnd ) {
				Wait();
			} else {
				Kill();
			}
		}
	}

	bool Create(bool suspend = false)
	{
		if ( m_hThread != NULL ) {
			return true;
		}
		m_hThread = CreateThread( NULL, 0, ThreadFunc, this, suspend ? CREATE_SUSPENDED : 0, &m_dwId );
		return m_hThread != NULL ? true : false;
	}

	void Resume()
	{
		if ( m_hThread != NULL ) {
			ResumeThread( m_hThread );
		}
	}

	void Wait()
	{
		if ( m_hThread != NULL ) {
			WaitForSingleObject( m_hThread, INFINITE );
		}
	}

	void Kill()
	{
		if ( m_hThread != NULL ) {
			TerminateThread( m_hThread, 0 );
		}
	}

	bool IsActive()
	{
		if ( m_hThread == NULL ) {
			return false;
		}
		DWORD dwCode;
		if ( GetExitCodeThread(m_hThread, &dwCode) ) {
			ResumeThread(m_hThread);
			return dwCode == STILL_ACTIVE;
		}
		return false;
	}

	static DWORD WINAPI ThreadFunc(LPVOID param)
	{
		CThread *thread = static_cast<CThread*>(param);
		thread->Run();
		ExitThread(0);
	}

	virtual void Run() = 0;

};
